@@ -71,6 +71,11 @@ where
             && pt.y <= self.position.y + self.size.y
     }
 
+    pub fn center(&self) -> Vector2<T> {
+        let two = T::one() + T::one();
+        self.position + Vector2::new(self.size.x / two, self.size.y / two)
+    }
+
     /// Extends rect to contain given point.
     ///
     /// # Notes
@@ -114,12 +119,13 @@ where
         let mut clipped = *self;
 
         if clipped.position.x < other.position.x {
-            clipped.position.x = other.position.x;
             clipped.size.x -= other.position.x - clipped.position.x;
+            clipped.position.x = other.position.x;
         }
+
         if clipped.position.y < other.position.y {
-            clipped.position.y = other.position.y;
             clipped.size.y -= other.position.y - clipped.position.y;
+            clipped.position.y = other.position.y;
         }
 
         let clipped_right_bottom = clipped.right_bottom_corner();
@@ -239,6 +245,37 @@ where
     pub fn y(&self) -> T {
         self.position.y
     }
+
+    #[inline]
+    #[must_use]
+    pub fn transform(&self, matrix: &Matrix3<T>) -> Self {
+        let min = self.position;
+        let max = self.right_bottom_corner();
+
+        let translation = Vector2::new(matrix[6], matrix[7]);
+
+        let mut transformed_min = translation;
+        let mut transformed_max = translation;
+
+        for i in 0..2 {
+            for j in 0..2 {
+                let a = matrix[(i, j)] * min[j];
+                let b = matrix[(i, j)] * max[j];
+                if a < b {
+                    transformed_min[i] += a;
+                    transformed_max[i] += b;
+                } else {
+                    transformed_min[i] += b;
+                    transformed_max[i] += a;
+                }
+            }
+        }
+
+        Self {
+            position: transformed_min,
+            size: transformed_max - transformed_min,
+        }
+    }
 }
 
 impl<T> Visit for Rect<T>
@@ -246,14 +283,14 @@ where
     T: NumAssign + Scalar + Visit + PartialOrd + Copy + 'static,
 {
     fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
-        visitor.enter_region(name)?;
+        let mut region = visitor.enter_region(name)?;
 
-        self.position.x.visit("X", visitor)?;
-        self.position.y.visit("Y", visitor)?;
-        self.size.x.visit("W", visitor)?;
-        self.size.y.visit("H", visitor)?;
+        self.position.x.visit("X", &mut region)?;
+        self.position.y.visit("Y", &mut region)?;
+        self.size.x.visit("W", &mut region)?;
+        self.size.y.visit("H", &mut region)?;
 
-        visitor.leave_region()
+        Ok(())
     }
 }
 
@@ -695,13 +732,13 @@ impl TriangleDefinition {
 
 impl Visit for TriangleDefinition {
     fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
-        visitor.enter_region(name)?;
+        let mut region = visitor.enter_region(name)?;
 
-        self.0[0].visit("A", visitor)?;
-        self.0[1].visit("B", visitor)?;
-        self.0[2].visit("C", visitor)?;
+        self.0[0].visit("A", &mut region)?;
+        self.0[1].visit("B", &mut region)?;
+        self.0[2].visit("C", &mut region)?;
 
-        visitor.leave_region()
+        Ok(())
     }
 }
 
@@ -809,7 +846,7 @@ pub fn get_closest_point_triangle_set<P: PositionProvider>(
     closest_index
 }
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Visit)]
 pub struct SmoothAngle {
     /// Current angle in radians.
     pub angle: f32,
@@ -904,18 +941,6 @@ impl Default for SmoothAngle {
     }
 }
 
-impl Visit for SmoothAngle {
-    fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
-        visitor.enter_region(name)?;
-
-        self.angle.visit("Angle", visitor)?;
-        self.target.visit("Target", visitor)?;
-        self.speed.visit("Speed", visitor)?;
-
-        visitor.leave_region()
-    }
-}
-
 #[derive(Copy, Clone, Hash, PartialOrd, PartialEq, Ord, Eq)]
 pub enum RotationOrder {
     XYZ,
@@ -1060,6 +1085,13 @@ pub fn vector_to_quat(vec: Vector3<f32>) -> UnitQuaternion<f32> {
     }
 }
 
+#[inline]
+pub fn m4x4_approx_eq(a: &Matrix4<f32>, b: &Matrix4<f32>) -> bool {
+    a.iter()
+        .zip(b.iter())
+        .all(|(a, b)| (*a - *b).abs() <= 0.001)
+}
+
 #[cfg(test)]
 mod test {
     use crate::algebra::Vector2;
