@@ -38,13 +38,67 @@ use std::{
 
 const INVALID_GENERATION: u32 = 0;
 
+pub trait PayloadContainer: Sized {
+    type Element: Sized;
+
+    fn new_empty() -> Self;
+
+    fn new(element: Self::Element) -> Self;
+
+    fn is_some(&self) -> bool;
+
+    fn as_ref(&self) -> Option<&Self::Element>;
+
+    fn as_mut(&mut self) -> Option<&mut Self::Element>;
+
+    fn replace(&mut self, element: Self::Element) -> Option<Self::Element>;
+
+    fn take(&mut self) -> Option<Self::Element>;
+}
+
+impl<T> PayloadContainer for Option<T> {
+    type Element = T;
+
+    fn new_empty() -> Self {
+        Self::None
+    }
+
+    fn new(element: Self::Element) -> Self {
+        Self::Some(element)
+    }
+
+    fn is_some(&self) -> bool {
+        Option::is_some(self)
+    }
+
+    fn as_ref(&self) -> Option<&Self::Element> {
+        Option::as_ref(self)
+    }
+
+    fn as_mut(&mut self) -> Option<&mut Self::Element> {
+        Option::as_mut(self)
+    }
+
+    fn replace(&mut self, element: Self::Element) -> Option<Self::Element> {
+        Option::replace(self, element)
+    }
+
+    fn take(&mut self) -> Option<Self::Element> {
+        Option::take(self)
+    }
+}
+
 /// Pool allows to create as many objects as you want in contiguous memory
 /// block. It allows to create and delete objects much faster than if they'll
 /// be allocated on heap. Also since objects stored in contiguous memory block
 /// they can be effectively accessed because such memory layout is cache-friendly.
 #[derive(Debug)]
-pub struct Pool<T: Sized> {
-    records: Vec<PoolRecord<T>>,
+pub struct Pool<T, P = Option<T>>
+where
+    T: Sized,
+    P: PayloadContainer<Element = T>,
+{
+    records: Vec<PoolRecord<T, P>>,
     free_stack: Vec<u32>,
 }
 
@@ -81,6 +135,7 @@ impl<T: 'static> Inspect for Handle<T> {
                 step: None,
                 precision: None,
                 description: "Index of an object in a pool.".to_string(),
+                is_modified: false,
             },
             PropertyInfo {
                 owner_type_id: TypeId::of::<Self>(),
@@ -93,6 +148,7 @@ impl<T: 'static> Inspect for Handle<T> {
                 step: None,
                 precision: None,
                 description: "Generation of an object in a pool.".to_string(),
+                is_modified: false,
             },
         ]
     }
@@ -108,7 +164,7 @@ impl<T> Display for Handle<T> {
 }
 
 /// Type-erased handle.
-#[derive(Copy, Clone, Debug, Ord, PartialOrd, PartialEq, Eq, Hash, Inspect)]
+#[derive(Copy, Clone, Debug, Ord, PartialOrd, PartialEq, Eq, Hash, Inspect, Visit)]
 pub struct ErasedHandle {
     /// Index of object in pool.
     #[inspect(read_only)]
@@ -144,17 +200,6 @@ impl<T> From<Handle<T>> for ErasedHandle {
     }
 }
 
-impl Visit for ErasedHandle {
-    fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
-        visitor.enter_region(name)?;
-
-        self.index.visit("Index", visitor)?;
-        self.generation.visit("Generation", visitor)?;
-
-        visitor.leave_region()
-    }
-}
-
 impl ErasedHandle {
     pub fn none() -> Self {
         Self {
@@ -190,12 +235,12 @@ impl ErasedHandle {
 
 impl<T> Visit for Handle<T> {
     fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
-        visitor.enter_region(name)?;
+        let mut region = visitor.enter_region(name)?;
 
-        self.index.visit("Index", visitor)?;
-        self.generation.visit("Generation", visitor)?;
+        self.index.visit("Index", &mut region)?;
+        self.generation.visit("Generation", &mut region)?;
 
-        visitor.leave_region()
+        Ok(())
     }
 }
 
@@ -212,13 +257,17 @@ impl<T> Debug for Handle<T> {
 }
 
 #[derive(Debug)]
-struct PoolRecord<T: Sized> {
+struct PoolRecord<T, P = Option<T>>
+where
+    T: Sized,
+    P: PayloadContainer<Element = T>,
+{
     /// Generation number, used to keep info about lifetime. The handle is valid
     /// only if record it points to is of the same generation as the pool record.
     /// Notes: Zero is unknown generation used for None handles.
     generation: u32,
     /// Actual payload.
-    payload: Option<T>,
+    payload: P,
 }
 
 impl<T: PartialEq> PartialEq for PoolRecord<T> {
@@ -227,26 +276,30 @@ impl<T: PartialEq> PartialEq for PoolRecord<T> {
     }
 }
 
-impl<T> Default for PoolRecord<T> {
+impl<T, P> Default for PoolRecord<T, P>
+where
+    P: PayloadContainer<Element = T> + 'static,
+{
     fn default() -> Self {
         Self {
             generation: INVALID_GENERATION,
-            payload: None,
+            payload: P::new_empty(),
         }
     }
 }
 
-impl<T> Visit for PoolRecord<T>
+impl<T, P> Visit for PoolRecord<T, P>
 where
-    T: Visit + Default + 'static,
+    T: Visit + 'static,
+    P: PayloadContainer<Element = T> + Visit,
 {
     fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
-        visitor.enter_region(name)?;
+        let mut region = visitor.enter_region(name)?;
 
-        self.generation.visit("Generation", visitor)?;
-        self.payload.visit("Payload", visitor)?;
+        self.generation.visit("Generation", &mut region)?;
+        self.payload.visit("Payload", &mut region)?;
 
-        visitor.leave_region()
+        Ok(())
     }
 }
 
@@ -270,15 +323,16 @@ impl<T> PartialEq for Handle<T> {
     }
 }
 
-impl<T> Visit for Pool<T>
+impl<T, P> Visit for Pool<T, P>
 where
-    T: Default + Visit + 'static,
+    T: Visit + 'static,
+    P: PayloadContainer<Element = T> + Default + Visit + 'static,
 {
     fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
-        visitor.enter_region(name)?;
-        self.records.visit("Records", visitor)?;
-        self.free_stack.visit("FreeStack", visitor)?;
-        visitor.leave_region()
+        let mut region = visitor.enter_region(name)?;
+        self.records.visit("Records", &mut region)?;
+        self.free_stack.visit("FreeStack", &mut region)?;
+        Ok(())
     }
 }
 
@@ -326,7 +380,10 @@ impl<T> Handle<T> {
     }
 }
 
-impl<T> Default for Pool<T> {
+impl<T> Default for Pool<T>
+where
+    T: 'static,
+{
     fn default() -> Self {
         Self::new()
     }
@@ -356,7 +413,10 @@ impl<T: Clone> Clone for Pool<T> {
     }
 }
 
-impl<T> Pool<T> {
+impl<T, P> Pool<T, P>
+where
+    P: PayloadContainer<Element = T> + 'static,
+{
     #[inline]
     pub fn new() -> Self {
         Pool {
@@ -378,12 +438,12 @@ impl<T> Pool<T> {
         u32::try_from(self.records.len()).expect("Number of records overflowed u32")
     }
 
-    fn records_get(&self, index: u32) -> Option<&PoolRecord<T>> {
+    fn records_get(&self, index: u32) -> Option<&PoolRecord<T, P>> {
         let index = usize::try_from(index).expect("Index overflowed usize");
         self.records.get(index)
     }
 
-    fn records_get_mut(&mut self, index: u32) -> Option<&mut PoolRecord<T>> {
+    fn records_get_mut(&mut self, index: u32) -> Option<&mut PoolRecord<T, P>> {
         let index = usize::try_from(index).expect("Index overflowed usize");
         self.records.get_mut(index)
     }
@@ -439,7 +499,7 @@ impl<T> Pool<T> {
     ) -> Result<Handle<T>, T> {
         let index_usize = usize::try_from(index).expect("index overflowed usize");
         match self.records.get_mut(index_usize) {
-            Some(record) => match record.payload {
+            Some(record) => match record.payload.as_ref() {
                 Some(_) => Err(payload),
                 None => {
                     let position = self
@@ -457,7 +517,7 @@ impl<T> Pool<T> {
                     };
 
                     record.generation = generation;
-                    record.payload = Some(payload);
+                    record.payload = P::new(payload);
 
                     Ok(Handle::new(index, generation))
                 }
@@ -467,7 +527,7 @@ impl<T> Pool<T> {
                 for i in self.records_len()..index {
                     self.records.push(PoolRecord {
                         generation: 1,
-                        payload: None,
+                        payload: P::new_empty(),
                     });
                     self.free_stack.push(i);
                 }
@@ -480,7 +540,7 @@ impl<T> Pool<T> {
 
                 self.records.push(PoolRecord {
                     generation,
-                    payload: Some(payload),
+                    payload: P::new(payload),
                 });
 
                 Ok(Handle::new(index, generation))
@@ -531,7 +591,7 @@ impl<T> Pool<T> {
 
             let record = PoolRecord {
                 generation,
-                payload: Some(payload),
+                payload: P::new(payload),
             };
 
             self.records.push(record);
@@ -586,7 +646,7 @@ impl<T> Pool<T> {
 
             let record = PoolRecord {
                 generation,
-                payload: Some(payload),
+                payload: P::new(payload),
             };
 
             self.records.push(record);
@@ -607,7 +667,7 @@ impl<T> Pool<T> {
     pub fn borrow(&self, handle: Handle<T>) -> &T {
         if let Some(record) = self.records_get(handle.index) {
             if record.generation == handle.generation {
-                if let Some(ref payload) = record.payload {
+                if let Some(payload) = record.payload.as_ref() {
                     payload
                 } else {
                     panic!("Attempt to borrow destroyed object at {:?} handle.", handle);
@@ -650,7 +710,7 @@ impl<T> Pool<T> {
         let record_count = self.records.len();
         if let Some(record) = self.records_get_mut(handle.index) {
             if record.generation == handle.generation {
-                if let Some(ref mut payload) = record.payload {
+                if let Some(payload) = record.payload.as_mut() {
                     payload
                 } else {
                     panic!("Attempt to borrow destroyed object at {:?} handle.", handle);
@@ -828,7 +888,7 @@ impl<T> Pool<T> {
     where
         F: FnOnce(&T) -> Handle<T>,
     {
-        let this = unsafe { &mut *(self as *mut Pool<T>) };
+        let this = unsafe { &mut *(self as *mut Pool<T, P>) };
         let first = self.try_borrow_mut(handle);
         if let Some(first_object) = first.as_ref() {
             let second_handle = func(first_object);
@@ -869,6 +929,26 @@ impl<T> Pool<T> {
         }
     }
 
+    /// Tries to move object out of the pool using the given handle. Returns None if given handle
+    /// is invalid. After object is moved out if the pool, all handles to the object will become
+    /// invalid.
+    #[inline]
+    pub fn try_free(&mut self, handle: Handle<T>) -> Option<T> {
+        let index = usize::try_from(handle.index).expect("index overflowed usize");
+        self.records.get_mut(index).and_then(|record| {
+            if record.generation == handle.generation {
+                if let Some(payload) = record.payload.take() {
+                    self.free_stack.push(handle.index);
+                    Some(payload)
+                } else {
+                    None
+                }
+            } else {
+                None
+            }
+        })
+    }
+
     /// Moves an object out of the pool using the given handle with a promise that the object will be returned back.
     /// Returns pair (ticket, value). The ticket must be used to put the value back!
     ///
@@ -1094,7 +1174,7 @@ impl<T> Pool<T> {
     /// assert_eq!(*iter.next().unwrap(), 321);
     /// ```
     #[must_use]
-    pub fn iter(&self) -> PoolIterator<T> {
+    pub fn iter(&self) -> PoolIterator<T, P> {
         unsafe {
             PoolIterator {
                 ptr: self.records.as_ptr(),
@@ -1107,7 +1187,7 @@ impl<T> Pool<T> {
     /// Creates new pair iterator that iterates over filled records using pair (handle, payload)
     /// Can be useful when there is a need to iterate over pool records and know a handle of
     /// that record.
-    pub fn pair_iter(&self) -> PoolPairIterator<T> {
+    pub fn pair_iter(&self) -> PoolPairIterator<T, P> {
         PoolPairIterator {
             pool: self,
             current: 0,
@@ -1129,7 +1209,7 @@ impl<T> Pool<T> {
     /// assert_eq!(*iter.next().unwrap(), 321);
     /// ```
     #[must_use]
-    pub fn iter_mut(&mut self) -> PoolIteratorMut<T> {
+    pub fn iter_mut(&mut self) -> PoolIteratorMut<T, P> {
         unsafe {
             PoolIteratorMut {
                 ptr: self.records.as_mut_ptr(),
@@ -1142,7 +1222,7 @@ impl<T> Pool<T> {
     /// Creates new pair iterator that iterates over filled records using pair (handle, payload)
     /// Can be useful when there is a need to iterate over pool records and know a handle of
     /// that record.
-    pub fn pair_iter_mut(&mut self) -> PoolPairIteratorMut<T> {
+    pub fn pair_iter_mut(&mut self) -> PoolPairIteratorMut<T, P> {
         unsafe {
             PoolPairIteratorMut {
                 current: 0,
@@ -1177,11 +1257,11 @@ impl<T> Pool<T> {
         }
     }
 
-    fn end(&self) -> *const PoolRecord<T> {
+    fn end(&self) -> *const PoolRecord<T, P> {
         unsafe { self.records.as_ptr().add(self.records.len()) }
     }
 
-    fn begin(&self) -> *const PoolRecord<T> {
+    fn begin(&self) -> *const PoolRecord<T, P> {
         self.records.as_ptr()
     }
 
@@ -1202,7 +1282,10 @@ impl<T> Pool<T> {
     }
 }
 
-impl<T> FromIterator<T> for Pool<T> {
+impl<T> FromIterator<T> for Pool<T>
+where
+    T: 'static,
+{
     fn from_iter<C: IntoIterator<Item = T>>(iter: C) -> Self {
         let iter = iter.into_iter();
         let (lower_bound, upper_bound) = iter.size_hint();
@@ -1217,7 +1300,11 @@ impl<T> FromIterator<T> for Pool<T> {
     }
 }
 
-impl<T> Index<Handle<T>> for Pool<T> {
+impl<T, P> Index<Handle<T>> for Pool<T, P>
+where
+    T: 'static,
+    P: PayloadContainer<Element = T> + 'static,
+{
     type Output = T;
 
     fn index(&self, index: Handle<T>) -> &Self::Output {
@@ -1225,44 +1312,60 @@ impl<T> Index<Handle<T>> for Pool<T> {
     }
 }
 
-impl<T> IndexMut<Handle<T>> for Pool<T> {
+impl<T, P> IndexMut<Handle<T>> for Pool<T, P>
+where
+    T: 'static,
+    P: PayloadContainer<Element = T> + 'static,
+{
     fn index_mut(&mut self, index: Handle<T>) -> &mut Self::Output {
         self.borrow_mut(index)
     }
 }
 
-impl<'a, T> IntoIterator for &'a Pool<T> {
+impl<'a, T, P> IntoIterator for &'a Pool<T, P>
+where
+    P: PayloadContainer<Element = T> + 'static,
+{
     type Item = &'a T;
-    type IntoIter = PoolIterator<'a, T>;
+    type IntoIter = PoolIterator<'a, T, P>;
 
     fn into_iter(self) -> Self::IntoIter {
         self.iter()
     }
 }
 
-impl<'a, T> IntoIterator for &'a mut Pool<T> {
+impl<'a, T, P> IntoIterator for &'a mut Pool<T, P>
+where
+    P: PayloadContainer<Element = T> + 'static,
+{
     type Item = &'a mut T;
-    type IntoIter = PoolIteratorMut<'a, T>;
+    type IntoIter = PoolIteratorMut<'a, T, P>;
 
     fn into_iter(self) -> Self::IntoIter {
         self.iter_mut()
     }
 }
 
-pub struct PoolIterator<'a, T> {
-    ptr: *const PoolRecord<T>,
-    end: *const PoolRecord<T>,
+pub struct PoolIterator<'a, T, P>
+where
+    P: PayloadContainer<Element = T>,
+{
+    ptr: *const PoolRecord<T, P>,
+    end: *const PoolRecord<T, P>,
     marker: PhantomData<&'a T>,
 }
 
-impl<'a, T> Iterator for PoolIterator<'a, T> {
+impl<'a, T, P> Iterator for PoolIterator<'a, T, P>
+where
+    P: PayloadContainer<Element = T> + 'static,
+{
     type Item = &'a T;
 
     fn next(&mut self) -> Option<Self::Item> {
         unsafe {
             while self.ptr != self.end {
                 let current = &*self.ptr;
-                if let Some(ref payload) = current.payload {
+                if let Some(payload) = current.payload.as_ref() {
                     self.ptr = self.ptr.offset(1);
                     return Some(payload);
                 }
@@ -1274,19 +1377,22 @@ impl<'a, T> Iterator for PoolIterator<'a, T> {
     }
 }
 
-pub struct PoolPairIterator<'a, T> {
-    pool: &'a Pool<T>,
+pub struct PoolPairIterator<'a, T, P: PayloadContainer<Element = T>> {
+    pool: &'a Pool<T, P>,
     current: usize,
 }
 
-impl<'a, T> Iterator for PoolPairIterator<'a, T> {
+impl<'a, T, P> Iterator for PoolPairIterator<'a, T, P>
+where
+    P: PayloadContainer<Element = T>,
+{
     type Item = (Handle<T>, &'a T);
 
     fn next(&mut self) -> Option<Self::Item> {
         loop {
             match self.pool.records.get(self.current) {
                 Some(record) => {
-                    if let Some(payload) = &record.payload {
+                    if let Some(payload) = record.payload.as_ref() {
                         let handle = Handle::new(self.current as u32, record.generation);
                         self.current += 1;
                         return Some((handle, payload));
@@ -1299,20 +1405,26 @@ impl<'a, T> Iterator for PoolPairIterator<'a, T> {
     }
 }
 
-pub struct PoolIteratorMut<'a, T> {
-    ptr: *mut PoolRecord<T>,
-    end: *mut PoolRecord<T>,
+pub struct PoolIteratorMut<'a, T, P>
+where
+    P: PayloadContainer<Element = T>,
+{
+    ptr: *mut PoolRecord<T, P>,
+    end: *mut PoolRecord<T, P>,
     marker: PhantomData<&'a mut T>,
 }
 
-impl<'a, T> Iterator for PoolIteratorMut<'a, T> {
+impl<'a, T, P> Iterator for PoolIteratorMut<'a, T, P>
+where
+    P: PayloadContainer<Element = T> + 'static,
+{
     type Item = &'a mut T;
 
     fn next(&mut self) -> Option<Self::Item> {
         unsafe {
             while self.ptr != self.end {
                 let current = &mut *self.ptr;
-                if let Some(ref mut payload) = current.payload {
+                if let Some(payload) = current.payload.as_mut() {
                     self.ptr = self.ptr.offset(1);
                     return Some(payload);
                 }
@@ -1324,21 +1436,27 @@ impl<'a, T> Iterator for PoolIteratorMut<'a, T> {
     }
 }
 
-pub struct PoolPairIteratorMut<'a, T> {
-    ptr: *mut PoolRecord<T>,
-    end: *mut PoolRecord<T>,
+pub struct PoolPairIteratorMut<'a, T, P>
+where
+    P: PayloadContainer<Element = T>,
+{
+    ptr: *mut PoolRecord<T, P>,
+    end: *mut PoolRecord<T, P>,
     marker: PhantomData<&'a mut T>,
     current: usize,
 }
 
-impl<'a, T> Iterator for PoolPairIteratorMut<'a, T> {
+impl<'a, T, P> Iterator for PoolPairIteratorMut<'a, T, P>
+where
+    P: PayloadContainer<Element = T> + 'static,
+{
     type Item = (Handle<T>, &'a mut T);
 
     fn next(&mut self) -> Option<Self::Item> {
         unsafe {
             while self.ptr != self.end {
                 let current = &mut *self.ptr;
-                if let Some(ref mut payload) = current.payload {
+                if let Some(payload) = current.payload.as_mut() {
                     let handle = Handle::new(self.current as u32, current.generation);
                     self.ptr = self.ptr.offset(1);
                     self.current += 1;
@@ -1412,7 +1530,7 @@ mod test {
             data: String,
         }
 
-        let mut pool = Pool::new();
+        let mut pool = Pool::<Value>::new();
         let foobar = pool.spawn(Value {
             data: "Foobar".to_string(),
         });
@@ -1427,12 +1545,12 @@ mod test {
         assert_eq!(pool.handle_of(pool.borrow(baz)), baz);
     }
 
+    #[derive(Debug, Eq, PartialEq)]
+    struct Payload;
+
     #[test]
     fn pool_test_spawn_at() {
-        let mut pool = Pool::new();
-
-        #[derive(Debug, Eq, PartialEq)]
-        struct Payload;
+        let mut pool = Pool::<Payload>::new();
 
         assert_eq!(pool.spawn_at(2, Payload), Ok(Handle::new(2, 1)));
         assert_eq!(pool.spawn_at(2, Payload), Err(Payload));
@@ -1449,4 +1567,18 @@ mod test {
         assert_eq!(pool.spawn(Payload), Handle::new(1, 2));
         assert_eq!(pool.spawn(Payload), Handle::new(0, 2));
     }
+
+    #[test]
+    fn pool_test_try_free() {
+        let mut pool = Pool::<Payload>::new();
+
+        assert_eq!(pool.try_free(Handle::NONE), None);
+        assert_eq!(pool.free_stack.len(), 0);
+
+        let handle = pool.spawn(Payload);
+        assert_eq!(pool.try_free(handle), Some(Payload));
+        assert_eq!(pool.free_stack.len(), 1);
+        assert_eq!(pool.try_free(handle), None);
+        assert_eq!(pool.free_stack.len(), 1);
+    }
 }
